#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/24 18:23:19
@Author  :   ChenHao
@Description  :  工具模块
@Contact :   jerrychen1990@gmail.com
'''
from loguru import logger

from aifori.config import DEFAULT_VOICE_CHUNK_SIZE
from aifori.core import Message
from liteai.core import Voice


def show_stream_content(stream_content, batch_size: int = 10):
    full_content = ""
    tmp_chunk = ""
    for chunk in stream_content:
        tmp_chunk += chunk
        if len(tmp_chunk) > batch_size:
            logger.info(f"chunk={tmp_chunk}")
            full_content += tmp_chunk
            tmp_chunk = ""
    full_content += tmp_chunk
    logger.info(f"{full_content=}")
    return full_content


def show_message(message: Message):
    logger.info(f"message from {message.role}:")
    if isinstance(message.content, str):
        logger.info(message.content)
        return message.content
    else:
        return show_stream_content(message.content)


def voice2api_stream(voice: Voice, chunk_size=DEFAULT_VOICE_CHUNK_SIZE):
    acc = b""
    for chunk in voice.byte_stream:
        acc += chunk
        while len(acc) > chunk_size:
            yield dict(voice_chunk=acc[:chunk_size])
            acc = acc[chunk_size:]
    if len(acc) > 0:
        yield dict(voice_chunk=acc)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/17 15:38:54
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''


import itertools
import json
import os
from typing import Callable, Dict, Iterable, List, Tuple
import requests
from aifori.core import AssistantMessage, ChatRequest, ChatSpeakRequest, Message, SpeakRequest
from loguru import logger

from liteai.core import ToolCall, Voice
from liteai.voice import build_voice, play_voice
from snippets import jdumps
from cachetools import LRUCache, cached

# urllib3.disable_warnings()


def handle_chat_stream(stream: Iterable[dict]):
    for chunk in stream:
        k, v = list(chunk.items())[0]
        logger.debug(f"receive chunk with type:{k}, size:{len(v)}, {v[:10]}")


def decode_message(stream: Iterable[dict | bytes], assistant_id: str) -> AssistantMessage:
    dict_stream = (e if isinstance(e, dict) else eval(e.decode("utf8")) for e in stream)
    content_stream, tool_stream = itertools.tee(dict_stream)
    content_stream = (e["text_chunk"] for e in content_stream if "text_chunk" in e)
    tool_calls = []
    for tool in tool_stream:
        if "tool_call" in tool:
            tool_call = json.loads(tool["tool_call"])
            logger.debug(f"decode tool call: {tool_call}")
            tool_calls.append(ToolCall.model_validate(tool_call))

    return AssistantMessage(user_id=assistant_id, content=content_stream, tool_calls=tool_calls)


def decode_voice(byte_stream: Iterable[dict | bytes], local_file_path: str) -> Voice:
    byte_stream = (e if isinstance(e, dict) else eval(e.decode("utf8")) for e in byte_stream)
    voice_stream = (e["voice_chunk"] for e in byte_stream if e.get("voice_chunk"))
    voice_stream = (eval(e) if isinstance(e, str) else e for e in voice_stream)
    logger.debug(f"decode voice with {local_file_path=}")
    voice = build_voice(byte_stream=voice_stream, file_path=local_file_path)
    return voice


def decode_chunks(chunks: Iterable[dict], assistant_id: str, return_voice: bool, local_file_path: str) -> Tuple[AssistantMessage, Voice]:
    s1, s2 = itertools.tee(chunks)
    message = decode_message(s1, assistant_id)
    if return_voice:
        voice = decode_voice(s2, local_file_path)
    else:
        voice = None
    return message, voice


class AiForiClient(object):
    def __init__(self, host: str):
        self.host = host

    def _do_request(self, path, method="post", _json=dict(), stream=False, files=dict()):
        url = f"{self.host}{path}"
        logger.debug(f"{method.upper()} to {url} with json=\n{jdumps(_json)}, {files=}, {stream=}")
        if _json:
            resp = requests.request(method, url, json=_json, stream=stream, verify=False)
        else:
            resp = requests.request(method, url, files=files, stream=stream, verify=False)
        resp.raise_for_status()
        if not stream:
            if resp.json()["code"] != 200:
                raise Exception(resp.json()["msg"])
            logger.debug(f"resp: {resp.json()}")
            return resp.json()["data"]

        return resp

    def check_health(self) -> bool:
        resp = self._do_request("/health", method="get")
        assert resp["status"].upper() == "OK"

    def create_assistant(self, assistant_id: str, name: str, desc: str, model: str, voice_config: dict = dict()) -> dict:
        data = {'id': assistant_id, 'name': name, 'desc': desc, 'model': model, 'voice_config': voice_config}
        resp = self._do_request('/assistant/create', _json=data)
        return resp

    @cached(LRUCache(maxsize=128))
    def get_assistant(self, assistant_id: str):
        data = {'id': assistant_id}
        resp = self._do_request('/assistant/get', _json=data)
        return resp

    def get_or_create_assistant(self, assistant_id: str, name: str, desc: str, model: str, voice_config: dict = dict()) -> dict:
        try:
            return self.get_assistant(assistant_id)
        except Exception as e:
            return self.create_assistant(assistant_id, name, desc, model, voice_config)

    def delete_assistant(self, assistant_id: str) -> dict:
        data = {'id': assistant_id}
        resp = self._do_request('/assistant/delete', _json=data)
        return resp

    def create_user(self, user_id: str, name: str, desc: str) -> dict:
        data = {'id': user_id, 'name': name, 'desc': desc}
        resp = self._do_request('/user/create', _json=data)
        return resp

    @cached(LRUCache(maxsize=128))
    def get_user(self, user_id: str):
        data = {'id': user_id}
        resp = self._do_request('/user/get', _json=data)
        return resp

    def get_or_create_user(self, user_id: str, name: str, desc: str) -> dict:
        try:
            return self.get_user(user_id)
        except Exception as e:
            return self.create_user(user_id, name, desc)

    def delete_user(self, user_id: str) -> dict:
        data = {'id': user_id}
        resp = self._do_request('/user/delete', _json=data)
        return resp

    def chat(self, chat_request: ChatRequest, stream=True) -> AssistantMessage:
        data = chat_request.model_dump()
        if stream:
            resp = self._do_request('/assistant/chat_stream', _json=data, stream=True).iter_lines()
            assistant_id = data["assistant_id"]
            return decode_message(resp, assistant_id)
        else:
            resp = self._do_request('/assistant/chat', _json=data)
            return AssistantMessage.model_validate(resp)

    def speak(self, speak_req: SpeakRequest, play=False, local_voice_path: str = None):
        resp = self._do_request('/assistant/speak_stream',
                                _json=speak_req.model_dump(), stream=True).iter_lines()
        voice = decode_voice(resp, local_voice_path)
        if play:
            play_voice(voice)
        return voice

    def chat_speak_stream(self, chat_speak_request: ChatSpeakRequest, local_voice_path: str = None, play: bool = False) -> Tuple[AssistantMessage, Voice]:
        resp = self._do_request('/assistant/chat_speak_stream', _json=chat_speak_request.model_dump(), stream=True).iter_lines()
        resp = (json.loads(item.decode("utf-8")) for item in resp)
        message, voice = decode_chunks(resp, assistant_id=chat_speak_request.assistant_id,
                                       return_voice=chat_speak_request.return_voice, local_file_path=local_voice_path)
        if play:
            play_voice(voice)
        return message, voice

    def play_music(self, user_id: str, music_desc: str, local_voice_path: str = None, play: bool = True, max_seconds=None) -> Voice:
        resp = self._do_request('/assistant/play_music_stream', _json=dict(user_id=user_id, music_desc=music_desc), stream=True).iter_lines()
        voice = decode_voice(resp, local_voice_path)
        if play:
            play_voice(voice, max_seconds=max_seconds)
        return voice

    def on_play_music(self, message, tool_call, **kwargs):
        logger.debug(f"play music with {tool_call=}, {kwargs=}")
        new_kwargs = dict(**tool_call.parameters, **kwargs)
        logger.debug(f"play music with {new_kwargs=}")
        self.play_music(**new_kwargs)

    def on_tool(self, message: AssistantMessage, callbacks=Dict[str, Callable], **kwargs):
        if tool_calls := message.tool_calls:
            for tool_call in tool_calls:
                if tool_call.name in callbacks:
                    logger.debug(f"on tool call: {tool_call}")
                    callbacks[tool_call.name](message=message, tool_call=tool_call, **kwargs)

    def clear_session(self, session_id: str) -> dict:
        resp = self._do_request('/session/clear', _json={'session_id': session_id})
        return resp

    def list_messages(self, assistant_id: str,  session_id: str = None, limit: int = 10) -> List[Message]:
        resp = self._do_request('/message/list', _json={'assistant_id': assistant_id, 'session_id': session_id, 'limit': limit})
        return [Message.model_validate(message) for message in resp]

    def update_rule(self, rule_path: str) -> str:
        if not os.path.exists(rule_path):
            raise FileNotFoundError(f"{rule_path} not found")
        if not rule_path.endswith(".jsonl"):
            raise ValueError(f"{rule_path} must be a jsonline file")

        with open(rule_path, "rb") as f:
            files = {'upload_file': (os.path.basename(rule_path), f)}
            resp = self._do_request('/rule/update', files=files)
            return resp



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/08 16:53:50
@Author  :   ChenHao
@Description  :   core classes
@Contact :   jerrychen1990@gmail.com
'''


import os
from typing import List, Optional
from pydantic import BaseModel, Field
from aifori.config import AGENT_DIR, DEFAULT_TEXT_CHUNK_SIZE, DEFAULT_TTS_TEXT_CHUNK_SIZE, DEFAULT_VOICE_CHUNK_SIZE, DEFAULT_VOICE_CONFIG
from liteai.core import LLMGenConfig, Message, ToolCall
import uuid
from snippets import dump, load
from loguru import logger


class UserMessage(Message):
    role: str = "user"


class AssistantMessage(Message):
    role: str = "assistant"
    tool_calls: Optional[List[ToolCall]] = Field(default=[], description="tool calls")

    @property
    def is_stream(self):
        return not isinstance(self.content, str)


class Memory(BaseModel):
    def add_message(self, message: Message):
        raise NotImplementedError

    def to_llm_messages(self) -> List[Message]:
        raise NotImplementedError


class BaseUser:
    role = "UNK"

    def __init__(self, name: str, desc: str, id: str = None, *args, **kwargs):
        self.id = id if id else str(uuid.uuid4())
        self.name = name
        self.desc = desc

    def get_config(self):
        return dict(name=self.name, desc=self.desc, id=self.id, role=self.role)

    @classmethod
    def get_config_path(cls, id: str):
        return os.path.join(AGENT_DIR, cls.role, f"{id}.json")

    def save(self):
        config_path = self.get_config_path(self.id)
        config = self.get_config()
        logger.info(f"save {self.role}'s {config} to {config_path}")
        dump(config, config_path)
        return config_path

    @classmethod
    def from_config(cls, id: str):
        config_path = cls.get_config_path(id)
        if not os.path.exists(config_path):
            raise FileNotFoundError(f"{cls.role}'s config for {id} not found!")
        data = load(config_path)
        logger.debug(f"load {cls.role} from {config_path}")
        return cls(**data)

    @classmethod
    def delete(cls, id: str):
        config_path = cls.get_config_path(id)
        if os.path.exists(config_path):
            logger.info(f"delete {cls.role} from {config_path}")
            os.remove(config_path)
        else:
            logger.warning(f"{cls.role} config file: {config_path} not exists, will not delete")


class ChatRequest(BaseModel):
    assistant_id: str = Field(description="AIAgent的ID,唯一键", examples=["test_assistant"])
    user_id: str = Field(description="用户的ID,唯一键", examples=["test_user"])
    session_id: str = Field(default=None, description="对话的ID,唯一键", examples=["test_session"])
    message: str = Field(description="用户发送的消息", examples=["你好呀，你叫什么名字？"])
    model: Optional[str] = Field(default=None, description="模型的ID,唯一键")
    do_remember: bool = Field(default=False, description="Agent是否记忆这轮对话")
    recall_memory: bool = Field(default=False, description="是否唤起长期记忆")
    llm_gen_config: LLMGenConfig = Field(default=LLMGenConfig(), description="llm的配置")


class SpeakRequest(BaseModel):
    assistant_id: str = Field(description="AIAgent的ID,唯一键", examples=["test_assistant"])
    message: str = Field(description="用户发送的消息", examples=["你好呀，你叫什么名字？"])
    voice_path: Optional[str] = Field(default=None, description="音频文件存储路径", examples=[None])
    voice_config: dict = Field(default=DEFAULT_VOICE_CONFIG, description="tts的配置")
    voice_chunk_size: int = Field(default=DEFAULT_VOICE_CHUNK_SIZE, description="默认音频字节chunk(byte)大小, mp3格式")


class ChatSpeakRequest(ChatRequest):
    return_text: bool = Field(default=True, description="是否返回文字")
    text_chunk_size: int = Field(default=DEFAULT_TEXT_CHUNK_SIZE, description="返回的文字chunk(字符)的token大小")
    voice_config: dict = Field(default=DEFAULT_VOICE_CONFIG, description="tts的配置")
    return_voice: bool = Field(default=False, description="是否返回音频")
    tts_text_chunk_size: int = Field(default=DEFAULT_TTS_TEXT_CHUNK_SIZE, description="积累多少个字符请求一次tts")
    voice_chunk_size: int = Field(default=DEFAULT_VOICE_CHUNK_SIZE, description="默认音频字节chunk(byte)大小, mp3格式")
    voice_path: Optional[str] = Field(default=None, description="音频文件存储路径", examples=[None])



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/18 17:23:23
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''

import unittest
from aifori.agent import *
from snippets import set_logger
from aifori.session import SESSION_MANAGER


USER_ID = "ut_user"
ASSISTANT_ID = "ut_agent"
logger = set_logger("dev", __name__)


class TestSession(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        logger.info("start test session")
        cls.session_id = "ut_session"

    def test_message(self):
        SESSION_MANAGER.clear_session(self.session_id)

        SESSION_MANAGER.add_message(message=UserMessage(content="你好"),
                                    from_id=USER_ID,
                                    to_id=ASSISTANT_ID, to_role="agent", session_id=self.session_id)
        SESSION_MANAGER.add_message(message=AssistantMessage(content="你好呀，我叫Aifori"), from_id=ASSISTANT_ID,
                                    to_id=USER_ID, to_role="user", session_id=self.session_id)
        SESSION_MANAGER.add_message(message=UserMessage(content="1+1=2吗"), from_id=USER_ID,
                                    to_id=ASSISTANT_ID, to_role="agent", session_id=self.session_id)
        SESSION_MANAGER.add_message(message=AssistantMessage(content="是的，1+1=2"), from_id=ASSISTANT_ID,
                                    to_id=USER_ID, to_role="user", session_id=self.session_id)

        messages = SESSION_MANAGER.get_history(_from=[USER_ID, ASSISTANT_ID], to=[USER_ID, ASSISTANT_ID], limit=10)
        logger.info(f"{messages=}")
        self.assertEquals(4, len(messages))
        messages = SESSION_MANAGER.get_history(_from=ASSISTANT_ID, limit=10)
        logger.info(f"{messages=}")
        self.assertEquals(2, len(messages))

        messages = SESSION_MANAGER.get_history(_from=ASSISTANT_ID, to=ASSISTANT_ID, operator="or", limit=10)
        logger.info(f"{messages=}")
        self.assertEquals(4, len(messages))

        SESSION_MANAGER.clear_session(self.session_id)

        messages = SESSION_MANAGER.get_history(_from=ASSISTANT_ID, limit=10)
        logger.info(f"{messages=}")
        self.assertEquals(0, len(messages))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/19 13:51:53
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''

import unittest
from aifori.agent import *
from loguru import logger
from snippets import set_logger

AGENT_ID = "ut_agent_id"
USER_ID = "ut_user_id"


@unittest.skip("skip test memory")
class TestMemory(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        set_logger("dev", __name__)
        logger.info("start test memory")

    # def test_mem0(self):
    #     memory = Mem0Memory(agent_id=AGENT_ID)
    #     memory.add_message(message=UserMessage(user_id=USER_ID, content="你好，我叫Nobody"), to_id=AGENT_ID)
    #     memory.add_message(message=AssistantMessage(user_id=AGENT_ID, content="你好，我叫Aifori"), to_id=USER_ID)
    #     messages = memory.to_llm_messages()
    #     logger.info(messages)

    # def test_db_mem(self):
    #     memory = DBMemory(agent_id=AGENT_ID)
    #     memory.add_message(message=UserMessage(user_id=USER_ID, content="你好，我叫Nobody"), to_id=AGENT_ID)
    #     memory.add_message(message=AssistantMessage(user_id=AGENT_ID, content="你好，我叫Aifori"), to_id=USER_ID)
    #     messages = memory.to_llm_messages()
    #     logger.info(messages)



from os import curdir
import os
from loguru import logger

# 文件配置
AIFORI_ENV = os.getenv("AIFORI_ENV", "dev").upper()

AIFORI_HOME = os.getenv("AIFORI_HOME", os.path.abspath(os.path.dirname(curdir)))
logger.info(f"AIFORI_HOME: {AIFORI_HOME}")

VOICE_DIR = os.path.join(AIFORI_HOME, "voices")
os.makedirs(VOICE_DIR, exist_ok=True)

SESSION_DIR = os.path.join(AIFORI_HOME, "sessions")
os.makedirs(SESSION_DIR, exist_ok=True)

AGENT_DIR = os.path.join(AIFORI_HOME, "agents")
os.makedirs(AGENT_DIR, exist_ok=True)

DATA_DIR = os.path.join(AIFORI_HOME, "data")
os.makedirs(DATA_DIR, exist_ok=True)

RULE_DIR = os.path.join(AIFORI_HOME, "rule")
os.makedirs(RULE_DIR, exist_ok=True)
DEFAULT_RULE_NAME = "rule.jsonl"
DEFAULT_RULE_PATH = os.path.join(RULE_DIR, DEFAULT_RULE_NAME)

DB_DIR = os.path.join(AIFORI_HOME, "db")
os.makedirs(DATA_DIR, exist_ok=True)

MUSIC_DIR = os.path.join(AIFORI_HOME, "music")
os.makedirs(MUSIC_DIR, exist_ok=True)


# mem0配置
MEM_CONFIG = {
    "vector_store": {
        "provider": "qdrant",
        "config": {
            "host": "localhost",
            "port": 6333,
        }
    }
}
MEM_ON = False


# agent配置
DEFAULT_USER_NAME = "NoBody"
DEFAULT_USER_DESC = "一个空巢年轻人,没有朋友,没有爱人,没有工作,没有希望。喜欢音乐和诗歌，喜欢一切华美而哀伤的事物。"

DEFAULT_AI_NAME = "Aifori"
DEFAULT_AI_DESC = f"{DEFAULT_AI_NAME}是一款基于Hill助人理论的情感支持AI，拥有专业的心理咨询话术能力。富有同情心和同理心，说话简洁而幽默"

DEFAULT_MODEL = "GLM-4-Air"

DEFAULT_VOICE_CONFIG = dict(voice_id="tianxin_xiaoling", speed=1, pitch=0)

DEFAULT_TTS_TEXT_CHUNK_SIZE = 32
DEFAULT_TEXT_CHUNK_SIZE = 10
DEFAULT_VOICE_CHUNK_SIZE = 4096*10


DEFAULT_SYSTEM_TEMPLATE = """你是一个善解人意的聊天机器人，你需要参考**背景信息**和户聊天，聊天时需要遵守**聊天规范**
**背景信息**
你的名字:{agent_name}
你的人设:{agent_desc}
用户的名字:{user_name}
用户的人设:{user_desc}

**聊天规范**
1.请简短、温和地和用户对话
2.当用户问询你的名字时，请回答出你的名字"""


# LiteAI配置
LITE_AI_LOG_LEVEL = "DEBUG"

# 日志配置

LOG_HOME = os.getenv("AIFORI_LOG_HOME", os.path.join(AIFORI_HOME, "logs"))
os.makedirs(LOG_HOME, exist_ok=True)

# STD_FMT = LoguruFormat.DETAIL if AIFORI_ENV == "DEV" else LoguruFormat.SIMPLE
# STD_LEVEL = "DEBUG" if AIFORI_ENV == "DEV" else "INFO"
# def MODULE_FILTER(record): return __name__ in record["name"]
# def SERVICE_FILTER(record): return "service" in record["name"]


# FILE_RETENTION = "7 days" if AIFORI_ENV == "DEV" else "30 days"

# handlers = {
#     "aifori_std": dict(sink=sys.stdout, format=STD_FMT, colorize=True, level=STD_LEVEL, filter=lambda x: MODULE_FILTER(x) or SERVICE_FILTER(x), enqueue=True),
#     "aifori_output": dict(sink=os.path.join(LOG_HOME, "aifori_output.log"), format=LoguruFormat.File_SIMPLE, level="INFO",
#                           filter=MODULE_FILTER, enqueue=True, backtrace=True, colorize=True, rotation="00:00", retention=FILE_RETENTION),
#     "aifori_service_out": dict(sink=os.path.join(LOG_HOME, "service_output.log"), format=LoguruFormat.File_SIMPLE, level="INFO",
#                                filter=SERVICE_FILTER, enqueue=True, backtrace=True, colorize=True, rotation="00:00", retention=FILE_RETENTION)
# }
# if AIFORI_ENV == "DEV":
#     handlers.update({
#         "aifori_detail": dict(sink=os.path.join(LOG_HOME, "aifori_detail.log"), format=LoguruFormat.FILE_DETAIL, level="DEBUG",
#                               filter=MODULE_FILTER, enqueue=True, backtrace=True, colorize=True, rotation="00:00", retention=FILE_RETENTION),
#         "aifori_detail": dict(sink=os.path.join(LOG_HOME, "service_detail.log"), format=LoguruFormat.FILE_DETAIL, level="DEBUG",
#                               filter=SERVICE_FILTER, enqueue=True, backtrace=True, colorize=True, rotation="00:00", retention=FILE_RETENTION),
#     })

# LOG_CONFIG = {
#     "handlers": handlers
# }



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/08/13 14:37:13
@Author  :   ChenHao
@Description  :   音乐相关
@Contact :   jerrychen1990@gmail.com
'''
import os
from loguru import logger
from aifori.config import DEFAULT_VOICE_CHUNK_SIZE, MUSIC_DIR
from liteai.core import Parameter, ToolDesc, Voice
from liteai.voice import file2voice, play_voice


def get_music_voice(desc: str) -> Voice:
    logger.debug(f"search music with : {desc=}")
    music_path = os.path.join(MUSIC_DIR, "the_moon_song_sample.mp3")
    logger.debug(f"play music : {music_path}")
    voice = file2voice(music_path)
    return voice


MusicToolDesc = ToolDesc(name="play_music", description="根据描述播放音乐",
                         parameters=[Parameter(name="music_desc", description="音乐描述", type="string", required=True)],
                         content_resp="正在播放音乐")


if __name__ == "__main__":
    voice = get_music_voice("dnll")
    play_voice(voice, buffer_size=DEFAULT_VOICE_CHUNK_SIZE)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/02 17:07:28
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''


import copy
from loguru import logger
from aifori.config import *
from aifori.core import *
from aifori.memory import MEM, DBMemory
from liteai import api as liteai_api
from aifori.session import SESSION_MANAGER
from liteai.api import tts
from liteai.core import ToolDesc, Voice


class AIAgent(BaseUser):
    role = "assistant"

    def __init__(self, model: str, voice_config: dict = {}, tools: List[ToolDesc | dict] = [], **kwargs):
        super().__init__(**kwargs)
        self.memory = DBMemory(agent_id=self.id)
        self.model = model
        self.voice_config = voice_config
        self.system_template = DEFAULT_SYSTEM_TEMPLATE
        self.tools = [e if isinstance(e, ToolDesc) else ToolDesc(**e) for e in tools]

    def _build_system(self, user_id: str) -> str:
        user = Human.from_config(id=user_id)

        system = self.system_template.format(agent_name=self.name, agent_desc=self.desc, user_name=user.name, user_desc=user.desc)
        logger.debug(f"{system=}")
        return system

    def _static_response(self, resp: str, stream: bool, **kwargs) -> Message:
        if stream:
            return AssistantMessage(content=(e for e in resp), user_id=self.id)
        return AssistantMessage(content=resp, user_id=self.id)

    def _get_memory(self, message: UserMessage) -> str:
        mems = MEM.search(query=message.content, user_id=message.user_id)
        logger.debug(f"get {mems=}")
        mems = [f"{idx}. {mem['text']}" for idx, mem in enumerate(mems, start=1)]
        mems = "可供参考的历史记忆:\n" + "\n".join(mems)
        return mems

    def chat(self, user_id: str, message: UserMessage,  session_id=None, recall_memory=False, stream=False,
             model: str = None,  ** kwargs) -> AssistantMessage:
        user = Human.from_config(id=user_id)
        history = SESSION_MANAGER.get_history(_from=[self.id, user_id], to=[self.id, user_id], session_id=session_id)
        meta = dict(user_info=user.desc, user_name=user.name, bot_info=self.desc, bot_name=self.name)

        system = self._build_system(user_id=user_id)

        user_content = message.content
        if recall_memory:
            mem = self._get_memory(message=message)
            user_content = mem+"\n"+user_content
        message = dict(role="user", content=user_content)
        messages = [dict(role="system", content=system)] + history + [message]

        kwargs["handle_system"] = False
        if model:
            logger.debug(f"chat with tmp model: {model}")
        else:
            model = self.model

        llm_resp = liteai_api.agent_chat(model=model, messages=messages, stream=stream, meta=meta,
                                         tools=self.tools, **kwargs, log_level=LITE_AI_LOG_LEVEL)

        return AssistantMessage(content=llm_resp.content, tool_calls=llm_resp.tool_calls)

    def speak(self, message: Message | str, stream=True, voice_path: str = None, **kwargs) -> Voice:
        speak_config = copy.copy(self.voice_config)
        speak_config.update(**kwargs)

        if isinstance(message, Message):
            text = message.content
        else:
            text = message
        voice = tts(text=text, model="speech-01-turbo", stream=stream, tgt_path=voice_path, **kwargs, log_level=LITE_AI_LOG_LEVEL)
        return voice

    def get_config(self):
        config = super().get_config()
        config.update(model=self.model, tools=self.tools)
        return config


class Human(BaseUser):
    role = "user"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/08/09 11:09:54
@Author  :   ChenHao
@Description  :   测试aifori的api函数
@Contact :   jerrychen1990@gmail.com
'''


import unittest

from aifori.api import *
from aifori.client import decode_chunks
from aifori.core import ChatRequest
# from aifori.client import handle_chat_stream
from aifori.music import MusicToolDesc
from aifori.util import show_message
from liteai.core import LLMGenConfig
from liteai.tool import CurrentContextToolDesc
from liteai.voice import play_file, play_voice
from snippets.logs import set_logger


class APITestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        set_logger("dev", __name__)
        logger.info("start test assistant")
        cls.assistant_id = "ut_assistant"
        cls.user_id = "ut_user"
        cls.session_id = "ut_session"
        cls.tools = [CurrentContextToolDesc]

    def test_assistant1(self):
        # Test creating a new assistant
        assistant = create_assistant(name=self.assistant_id, desc=self.assistant_id, id=self.assistant_id, model=DEFAULT_MODEL,
                                     do_save=True, tools=self.tools)
        self.assertIsNotNone(assistant)

    def test_create_user2(self):
        # Test creating a new user
        user = create_user(name=self.user_id, desc=self.user_id, id=self.user_id, do_save=True)
        self.assertIsNotNone(user)

    def test_chat_assistant3(self):
        create_assistant(name=self.assistant_id, desc=self.assistant_id, id=self.assistant_id, model=DEFAULT_MODEL,
                         do_save=True, tools=self.tools)
        create_user(name=self.user_id, desc=self.user_id, id=self.user_id, do_save=True)

        req = ChatRequest(assistant_id=self.assistant_id, user_id=self.user_id, session_id=self.session_id,
                          message="你好，你叫什么名字", do_remember=True)
        message = chat_assistant(req, stream=False)
        show_message(message)
        assert "ut_assistant" in message.content.lower()

        req = ChatRequest(assistant_id=self.assistant_id, user_id=self.user_id, session_id=self.session_id,
                          message="推荐三首歌", do_remember=True)
        message = chat_assistant(req, stream=True)
        show_message(message)

        # 测试多轮
        req = ChatRequest(assistant_id=self.assistant_id, user_id=self.user_id, session_id=self.session_id,
                          message="介绍一下第三首", do_remember=False)
        message = chat_assistant(req, stream=False)
        show_message(message)

        # 测试工具调用
        req = ChatRequest(assistant_id=self.assistant_id, user_id=self.user_id, session_id=self.session_id,
                          message="今天是几号呀，星期几？", do_remember=True)
        message = chat_assistant(req, stream=True)
        show_message(message)

    def test_speak_assistant(self):
        create_assistant(name=self.assistant_id, desc=self.assistant_id, id=self.assistant_id, model=DEFAULT_MODEL, do_save=True)
        voice_path = "./tmp/ut_test_api_speak.mp3"

        req = SpeakRequest(assistant_id=self.assistant_id, message="你好，我叫Aifori，有什么可以帮你的么", voice_path=voice_path)
        voice: Voice = speak_assistant(req, stream=True)
        logger.info(f"{voice.file_path=}")
        play_voice(voice)
        assert os.path.exists(voice.file_path)
        play_file(voice.file_path)

        voice_path = "./tmp/ut_test_api_speak_reply.mp3"

        req = SpeakRequest(assistant_id=self.assistant_id, message="你好，我是Nobody",
                           voice_config=dict(voice_id="junlang_nanyou", speed=.6, pitch=-4), voice_path=voice_path)
        voice: Voice = speak_assistant(req, stream=False)
        logger.info(f"{voice.file_path=}")
        assert os.path.exists(voice.file_path)
        play_voice(voice)

    def test_chat_music(self):

        tools = [MusicToolDesc]

        assistant = create_assistant(id="ut_tool_assistant", name="ut_tool_assistant",
                                     desc="ut_tool_assistant", model=DEFAULT_MODEL, exists_ok=True, tools=tools)
        create_user(name=self.user_id, desc=self.user_id, id=self.user_id, do_save=True)

        req = ChatRequest(assistant_id=assistant.id, user_id=self.user_id, session_id=self.session_id,
                          message="推荐一首欢快的歌曲", do_remember=True)
        message = chat_assistant(req, stream=False)

        show_message(message)

    def test_chat_speak_assistant(self):
        create_assistant(name=self.assistant_id, desc=self.assistant_id, id=self.assistant_id, model=DEFAULT_MODEL, do_save=True)
        create_user(name=self.user_id, desc=self.user_id, id=self.user_id, do_save=True)

        clear_session(self.session_id)

        return_voice = False
        req = ChatSpeakRequest(assistant_id=self.assistant_id, user_id=self.user_id, session_id=self.session_id,
                               message="你好，你叫什么名字", stream=True, do_remember=False, return_text=True, return_voice=return_voice)
        resp = chat_speak_assistant(req)
        message, voice = decode_chunks(resp, req.assistant_id, return_voice, local_file_path=None)
        message_content = show_message(message)
        assert "ut_assistant" in message_content.lower()
        assert voice is None

        return_voice = True
        server_voice_path = "./tmp/ut_test_api_chat_speak_server.mp3"
        local_voice_path = "./tmp/ut_test_api_chat_speak_local.mp3"
        req = ChatSpeakRequest(assistant_id=self.assistant_id, user_id=self.user_id, session_id=self.session_id,
                               message="作一首五言绝句", stream=True, do_remember=False, return_text=True, return_voice=return_voice,
                               voice_path=server_voice_path, llm_gen_config=LLMGenConfig(max_tokens=16))

        resp = chat_speak_assistant(req)
        message, voice = decode_chunks(resp, req.assistant_id, return_voice, local_voice_path)

        play_voice(voice)
        assert os.path.exists(voice.file_path)
        play_file(voice.file_path)

        assert os.path.exists(server_voice_path)
        play_file(server_voice_path)

    def test_rule_chat(self):
        # 测试规则问答
        create_assistant(name=self.assistant_id, desc=self.assistant_id, id=self.assistant_id, model=DEFAULT_MODEL, do_save=True)
        create_user(name=self.user_id, desc=self.user_id, id=self.user_id, do_save=True)

        req = ChatRequest(assistant_id=self.assistant_id, user_id=self.user_id, session_id=self.session_id,
                          message="我叫神尼名字", do_remember=False)
        message = chat_assistant(req, stream=True)

        message = show_message(message)
        self.assertEquals("你叫Nobody", message)

    def test_clear10(self):
        delete_assistant(id=self.assistant_id)
        with self.assertRaises(FileNotFoundError) as context:
            get_assistant(id=self.assistant_id)

        delete_user(id=self.user_id)
        with self.assertRaises(FileNotFoundError) as context:
            get_user(id=self.user_id)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/15 18:21:56
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''
import datetime
import json
import os
import sqlite3
from typing import List


from aifori.config import DB_DIR
from loguru import logger

from sqlalchemy import DateTime, create_engine, Column, Integer, String

from sqlalchemy import Column, Integer, String
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from aifori.core import Message


# SQLAlchemy ORM 基础类
Base = declarative_base()

# 定义 SQLAlchemy ORM 模型


class MessageORM(Base):
    __tablename__ = 'Message'
    id = Column(Integer, primary_key=True, index=True)
    session_id = Column(String, index=True)
    content = Column(String)
    from_role = Column(String, index=True)
    to_role = Column(String, index=True)
    from_id = Column(String, index=True)
    to_id = Column(String, index=True)
    tool_calls = Column(String)
    create_datetime = Column(DateTime, default=datetime.datetime.now())

    @classmethod
    def from_message(cls, message: Message, to_id: str, to_role: str):
        orm_message = cls(content=message.content, from_id=message.user_id, to_id=to_id, from_role=message.role, to_role=to_role)
        return orm_message

    def to_message(self) -> Message:
        return Message(content=self.content, role=self.from_role,  user_id=self.from_id)

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "session_id": self.session_id,
            "from_role": self.from_role,
            "from_id": self.from_id,
            "content": self.content,
            "to_role": self.to_role,
            "to_id": self.to_id,
            "create_datetime": self.create_datetime
        }

    def __repr__(self):
        return json.dumps(self.to_dict(), ensure_ascii=False)


def get_db_session():
    db_file = os.path.join(DB_DIR, "aifori.db")
    if not os.path.exists(db_file):
        os.makedirs(os.path.dirname(db_file), exist_ok=True)

    engine = create_engine(f'sqlite:///{db_file}')

    Base.metadata.create_all(bind=engine)

    # 创建数据库会话
    SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)
    db = SessionLocal()
    return db


DB = get_db_session()


def _get_connection():
    db_file = os.path.join(DB_DIR, "aifori.db")
    if not os.path.exists(db_file):
        os.makedirs(os.path.dirname(db_file), exist_ok=True)
    return sqlite3.connect(db_file)


def do_query(sql: str) -> List[dict] | dict:
    try:
        con = _get_connection()
        cur = con.cursor()
        logger.debug(f"query with {sql=}")
        cur.execute(sql)
        items = cur.fetchall()
        return items
    except Exception as e:
        logger.exception(e)
        raise e


def do_execute(sql: str):
    try:
        con = _get_connection()
        cur = con.cursor()
        logger.debug(f"execute with {sql=}")
        cur.execute(sql)
        con.commit()
    except Exception as e:
        logger.exception(e)
        raise e


# def add_message(message: Message):
#     sql = f"""insert into MESSAGE(ID,AGENT_ID,USER_ID,CONTENT) values ('{message.user_id}','{message.agent_id}','{message.user_id}','{message.content}')"""


def init_db():
    agent_sql = """
CREATE TABLE IF NOT EXISTS JOB(
    ID varchar PRIMARY KEY,
    STATUS TEXT NOT NULL,
    VOICE_CONFIG TEXT NOT NULL,
    CREATE_TIME TIMESTAMP DEFAULT (datetime('now','localtime')),
    UPDATE_TIME TIMESTAMP DEFAULT (datetime('now','localtime'))
)
    """
    message_sql = """
CREATE TABLE IF NOT EXISTS MESSAGE(
    ID varchar PRIMARY KEY,
    AGENT_ID TEXT NOT NULL,
    USER_ID TEXT NOT NULL,
    CONTENT TEXT NOT NULL,
    CREATE_TIME TIMESTAMP DEFAULT (datetime('now','localtime')),
    UPDATE_TIME TIMESTAMP DEFAULT (datetime('now','localtime'))
)
    """

    do_execute(agent_sql)
    do_execute(message_sql)


if __name__ == "__main__":
    pass
    # init_db()
    # add_message(AssistantMessage(user_id="123",  content="789"))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/08 17:47:34
@Author  :   ChenHao
@Description  :   api层
@Contact :   jerrychen1990@gmail.com
'''
from threading import Thread
from typing import Iterable, List


from aifori.agent import AIAgent, Human
from aifori.buffer import Buffer
from aifori.config import *
from aifori.core import AssistantMessage, ChatRequest, ChatSpeakRequest, SpeakRequest, UserMessage
from aifori.music import get_music_voice
from aifori.rule import rule_match
from aifori.session import SESSION_MANAGER
from liteai.core import ToolDesc, Voice
from liteai.voice import dump_voice_stream
from snippets.utils import add_callback2gen


def clear_session(session_id: str):
    logger.info(f"clear session {session_id}")
    SESSION_MANAGER.clear_session(session_id)


def get_assistant(id: str) -> AIAgent:
    agent = AIAgent.from_config(id=id)
    return agent


def get_user(id: str) -> Human:
    user = Human.from_config(id=id)
    return user


def create_assistant(name: str, desc: str, id: str, model: str,
                     tools: List[ToolDesc] = [],
                     voice_config: dict = DEFAULT_VOICE_CONFIG, exists_ok=True, do_save=True) -> AIAgent:
    try:
        agent = get_assistant(id)
        msg = f"assistant:{id} already exists, will not create new one"
        logger.warning(msg)
        if not exists_ok:
            raise Exception(msg)
        else:
            return agent
    except Exception as e:
        pass

    agent = AIAgent(name=name, desc=desc, model=model, id=id, tools=tools, voice_config=voice_config)
    if do_save:
        agent.save()
    return agent


def create_user(name: str, desc: str, id: str, exists_ok=True, do_save=True) -> Human:
    try:
        user = get_user(id)
        msg = f"user:{id} already exists, will not create new one"
        logger.warning(msg)
        if not exists_ok:
            raise Exception(msg)
        else:
            return user
    except Exception as e:
        pass

    user = Human(name=name, desc=desc, id=id)
    if do_save:
        user.save()
    return user


def delete_assistant(id: str):
    return AIAgent.delete(id)


def delete_user(id: str):
    return Human.delete(id)


def do_remember(user_message: UserMessage, user_id: str, assistant_message: AssistantMessage, assistant_id: str, session_id: str):
    SESSION_MANAGER.add_message(user_message, from_id=user_id, to_id=assistant_id, to_role="assistant", session_id=session_id)

    if assistant_message.is_stream:
        def _remember_callback(chunks: List[str], **kwargs):
            message = AssistantMessage(content="".join(chunks))
            SESSION_MANAGER.add_message(message, from_id=assistant_id, to_id=user_id, to_role="user", session_id=session_id)
        assistant_message.content = add_callback2gen(assistant_message.content, _remember_callback)
    else:
        SESSION_MANAGER.add_message(assistant_message, from_id=assistant_id, to_id=user_id, to_role="user", session_id=session_id)


def static_response(response: str, stream: bool) -> AssistantMessage:
    if stream:
        return AssistantMessage(content=(e for e in response))
    return AssistantMessage(content=response)


def chat_assistant(chat_request: ChatRequest, stream=True, **kwargs) -> AssistantMessage:
    assistant = get_assistant(chat_request.assistant_id)
    user = get_user(chat_request.user_id)
    user_message = UserMessage(content=chat_request.message)
    session_id = chat_request.session_id
    # 添加规则
    match_rules = rule_match(query=user_message.content, match_all=False, regex=True)
    if match_rules:
        func = match_rules[0]["func"]
        if func == "static_response":
            response = match_rules[0]["kwargs"]["resp"]
            assistant_message = static_response(response=response, stream=stream)
    else:
        assistant_message = assistant.chat(user_id=user.id,
                                           message=user_message,  stream=stream, session_id=session_id, recall_memory=chat_request.recall_memory,
                                           model=chat_request.model,
                                           **chat_request.llm_gen_config.model_dump(), **kwargs)
    if chat_request.do_remember:
        do_remember(user_message, user.id, assistant_message, assistant.id, session_id)
    return assistant_message


def speak_assistant(speak_request: SpeakRequest, stream=True) -> Voice:
    assistant = get_assistant(speak_request.assistant_id)
    voice: Voice = assistant.speak(message=speak_request.message, stream=stream, voice_path=speak_request.voice_path,  **speak_request.voice_config)
    chunk_size = speak_request.voice_chunk_size

    def _chunk_stream(stream):
        acc = b""
        for chunk in stream:
            acc += chunk
            if len(acc) >= chunk_size:
                yield acc[:chunk_size]
                acc = acc[chunk_size:]
        if acc:
            yield acc
    voice.byte_stream = _chunk_stream(voice.byte_stream)
    return voice


def chat_speak_assistant(req: ChatSpeakRequest) -> Iterable[dict]:
    assistant = get_assistant(req.assistant_id)
    buffer = Buffer()

    def _chat(buffer: Buffer):
        agent_message = chat_assistant(ChatRequest(**req.model_dump()), stream=True)
        for chunk in agent_message.content:
            buffer.add_text(chunk)
        buffer.text_done = True

    def _speak(buffer: Buffer):
        for text_chunk in buffer.gen_tts(text_chunk_size=req.tts_text_chunk_size):
            logger.debug(f"speak: [{text_chunk}], with size {len(text_chunk)}")
            voice: Voice = assistant.speak(message=text_chunk, stream=True, **req.voice_config)
            for chunk in voice.byte_stream:
                buffer.add_voice(chunk)
        if req.voice_path:
            dump_voice_stream(buffer.voice_stream, req.voice_path)
        buffer.voice_done = True

    chat_thread = Thread(target=_chat, args=(buffer,))
    chat_thread.start()

    if req.return_voice:
        speak_thread = Thread(target=_speak, args=(buffer,))
        logger.debug("speak_thread start")
        # print("speak start")
        speak_thread.start()
    return buffer.gen(gen_text=req.return_text, text_chunk_size=req.text_chunk_size, gen_voice=req.return_voice, voice_chunk_size=req.voice_chunk_size)


def play_music(user_id: str, music_desc: str):
    user = get_user(user_id)
    logger.info("play music")
    voice: Voice = get_music_voice(music_desc)
    return voice



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/24 16:49:57
@Author  :   ChenHao
@Description  :   规则相关
@Contact :   jerrychen1990@gmail.com
'''

import os
import re
from typing import List

from aifori.config import DEFAULT_RULE_NAME, RULE_DIR
from snippets import load_with_cache
from loguru import logger


def rule_match(query: str, rule_file_name: str = DEFAULT_RULE_NAME, regex=False, match_all=True) -> List[dict]:
    file_path = os.path.join(RULE_DIR, rule_file_name)
    rules = load_with_cache(file_path)
    logger.debug(f"check {len(rules)} rules for query: {query}")
    rs = []

    for rule in rules:
        key = rule["key"]
        if isinstance(key, list):
            if any(k in query for k in key):
                rs.append(rule)
                if not match_all:
                    break
        else:
            if not regex:
                if isinstance(key, str) and key in query:
                    rs.append(rule)
                    if not match_all:
                        break
            else:
                if re.search(key, query):
                    rs.append(rule)
                    if not match_all:
                        break

    return rs



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/08/15 10:13:04
@Author  :   ChenHao
@Description  :   测试music
@Contact :   jerrychen1990@gmail.com
'''
import unittest

from aifori.api import *
# from aifori.client import handle_chat_stream
from aifori.music import get_music_voice
from liteai.voice import play_voice
from snippets.logs import set_logger


class TestMusic(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        set_logger("dev", __name__)
        logger.info("start test assistant")
        cls.assistant_id = "ut_assistant"
        cls.user_id = "ut_user"
        cls.session_id = "ut_session"

    def test_play_music(self):
        voice: Voice = get_music_voice("dnll")
        play_voice(voice)



# tasks.py
import datetime
from celery import Celery
from loguru import logger
from aifori.memory import MEM

# 创建 Celery 应用，指定 Redis 作为消息代理
app = Celery('task', broker='redis://localhost:6379/0')


@app.task
def add_message2memory(message: dict):
    logger.debug(f"executing task add message2memory with {message}")
    mem_info = dict(data=message["content"])
    if message["from_role"] == "user":
        mem_info["user_id"] = message["from_id"]
    else:
        mem_info["agent_id"] = message["from_id"]

    logger.debug(f"add {mem_info=} to mem")
    MEM.add(**mem_info)
    logger.debug("add message to mem done")


if __name__ == "__main__":
    message = {'id': 186, 'session_id': '7ed9b796-1322-461a-bb5b-68deeee75184', 'from_role': 'agent', 'from_id': 'fe918725-ee28-4add-834b-46cb1ecaf56b',
               'content': '作为AI，我没有个人喜好，但我可以欣赏很多歌手的才华。如果你愿意分享，我很乐意听听你喜欢的歌手，也许我们可以一起探讨他们的音乐和诗歌。你最近有没有听什么让你印象深刻的歌曲？', 'to_role': 'user', 'to_id': '34d0f275-1e7c-4899-921c-94b6338856c1', 'create_datetime': datetime.datetime(2024, 7, 30, 17, 35, 36, 408470)}
    add_message2memory(message)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/19 15:32:20
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''

import os
from typing import List

from sqlalchemy import and_, desc, or_
from aifori.core import AssistantMessage, Message
from aifori.config import AIFORI_ENV, LOG_HOME, MEM_ON
from aifori.db import DB, MessageORM
from aifori.task import add_message2memory
from loguru import logger
from snippets import set_logger, jdumps


logger = set_logger(AIFORI_ENV, __name__,  std=False, log_dir=os.path.join(LOG_HOME, "session"))


class SessionManager:

    def add_message(self, message: Message, from_id: str, to_id: str, to_role: str, session_id: str):
        message_info = dict(content=message.content, from_id=from_id, to_id=to_id, from_role=message.role,
                            to_role=to_role, session_id=session_id)
        if isinstance(message, AssistantMessage):
            message_info["tool_calls"] = jdumps(message.tool_calls)
        orm_message = MessageORM(**message_info)

        DB.add(orm_message)
        DB.commit()
        DB.refresh(orm_message)
        logger.info(f"add message {jdumps(orm_message.to_dict(), indent=None)}")
        if MEM_ON:
            add_message2memory.delay(orm_message.to_dict())

    def get_history(self, _from: str | List[str] = None, to: str | List[str] = None, operator="and", session_id: str = None, limit=10) -> List[Message]:
        logger.debug(f"get history with {_from=}. {to=}, {session_id=}, {limit=}")
        query = DB.query(MessageORM)
        if session_id:
            query = query.filter(MessageORM.session_id == session_id)
        filter_phase = None
        if _from:
            _from = [_from] if isinstance(_from, str) else _from
            filter_phase = MessageORM.from_id.in_(_from)
        if to:
            to = [to] if isinstance(to, str) else to
            tmp_phase = MessageORM.to_id.in_(to)

            if filter_phase is not None:
                if operator == "and":
                    filter_phase = and_(filter_phase, tmp_phase)
                else:
                    filter_phase = or_(filter_phase, tmp_phase)
            else:
                filter_phase = MessageORM.to_id.in_(to)

        if filter_phase is not None:
            query = query.filter(filter_phase)

        orm_messages: List[MessageORM] = query.order_by(desc(MessageORM.id)).limit(limit).all()
        orm_json_messages = [m.to_dict() for m in orm_messages]
        logger.debug(f"get history with {orm_json_messages=}")

        messages = [m.to_message() for m in orm_messages[::-1]]
        return messages

    def clear_session(self, session_id: str):
        logger.info(f"clear session with {session_id=}")
        DB.query(MessageORM).filter(MessageORM.session_id == session_id).delete()
        DB.commit()


SESSION_MANAGER = SessionManager()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/08/07 19:26:26
@Author  :   ChenHao
@Description  :  同步音频、文字的缓冲区
@Contact :   jerrychen1990@gmail.com
'''


from aifori.config import *


class Buffer:
    def __init__(self):
        self.voice_stream = b""
        self.text = ""
        self.text_output_offset = 0
        self.voice_output_offset = 0
        self.tts_output_offset = 0
        self.text_done = False
        self.voice_done = False

    def add_text(self, text: str):
        self.text += text

    def add_voice(self, voice: bytes):
        self.voice_stream += voice

    def gen_tts(self, text_chunk_size=DEFAULT_TTS_TEXT_CHUNK_SIZE):
        while not self.text_done:
            while len(self.text) - self.tts_output_offset > text_chunk_size:
                yield self.text[self.tts_output_offset:self.tts_output_offset+text_chunk_size]
                self.tts_output_offset += text_chunk_size
        if self.tts_output_offset < len(self.text):
            yield self.text[self.tts_output_offset:]

    def gen(self, gen_text=True, text_chunk_size=DEFAULT_TEXT_CHUNK_SIZE, gen_voice=False, voice_chunk_size=DEFAULT_VOICE_CHUNK_SIZE):
        def _build_chunk(key, content):
            logger.debug(f"yield {key} with size {len(content)}, sample:{content[:10]}")
            return {key: content}

        while (gen_text and not self.text_done) or (gen_voice and not self.voice_done):
            if gen_text:
                while len(self.text) - self.text_output_offset > text_chunk_size:
                    yield _build_chunk("text_chunk", self.text[self.text_output_offset:self.text_output_offset+text_chunk_size])
                    self.text_output_offset += text_chunk_size
            if gen_voice:
                while len(self.voice_stream) - self.voice_output_offset > voice_chunk_size:
                    yield _build_chunk("voice_chunk", self.voice_stream[self.voice_output_offset:self.voice_output_offset+voice_chunk_size])
                    self.voice_output_offset += voice_chunk_size

        if gen_text and self.text_output_offset < len(self.text):
            yield _build_chunk("text_chunk", self.text[self.text_output_offset:])
        if gen_voice and self.voice_output_offset < len(self.voice_stream):
            yield _build_chunk("voice_chunk", self.voice_stream[self.voice_output_offset:])



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/04 14:25:18
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''


import os
from snippets import set_logger
from dotenv import load_dotenv
from aifori.config import AIFORI_ENV, LOG_HOME


set_logger(AIFORI_ENV, __name__, log_dir=os.path.join(LOG_HOME, "aifori"), show_process=True)
load_dotenv()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/08 17:14:50
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''

import copy
from typing import List


from aifori.core import Memory, Message, Memory
from liteai.core import Message
from pydantic import Field
import mem0


class RawMemory(Memory):
    size: int = Field(default=10, description="The size of the memory")
    history: List[Message] = Field(default=[], description="The history of the memory")

    def to_llm_messages(self) -> List[Message]:
        return copy.copy(self.history)

    def add_message(self, message: Message) -> None:
        self.history.append(message)

    # def to_json(self) -> dict:
    #     logger.debug(f"history: {self.history}")
    #     return {"size": self.size, "history": self.history}


class DBMemory(Memory):
    agent_id: str = Field(description="The id of the agent")
    size: int = Field(default=10, description="The size of the memory")

    def to_llm_messages(self) -> List[Message]:
        return None


MEM = mem0.Memory()

# MEM = mem0.Memory.from_config(MEM_CONFIG)


# class Mem0Memory(Memory):
#     agent_id: str = Field(description="The id of the agent")
#     size: int = Field(default=10, description="The size of the memory")

#     def to_llm_messages(self) -> List[Message]:
#         messages = _MEM.get_all(agent_id=self.agent_id, limit=self.size)

#         logger.debug(f"{messages=}")
#         return messages

#     def add_message(self, message: Message, to_id: str) -> None:
#         user_id, agent_id = message.user_id, to_id
#         if message.role != "user":
#             user_id, agent_id = agent_id, user_id
#         logger.debug(f"add message {user_id=}, {agent_id=} {message.content=}")
#         rs = _MEM.add(message.content, user_id=user_id, agent_id=agent_id)
#         logger.info(rs)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Time    :   2024/07/04 15:14:54
@Author  :   ChenHao
@Description  :   
@Contact :   jerrychen1990@gmail.com
'''

import unittest


from aifori.agent import *
from aifori.api import create_assistant, create_user, get_assistant, clear_session
from loguru import logger
from liteai.tool import CurrentContextToolDesc
from aifori.config import *
from aifori.music import MusicToolDesc
from aifori.util import show_message
from snippets import set_logger
from liteai.core import Voice
from liteai.voice import play_voice


class TestAssistant(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        set_logger("dev", __name__)
        logger.info("start test assistant")
        cls.assistant_id = "ut_assistant"
        cls.user_id = "ut_user"
        cls.assistant = create_assistant(id=cls.assistant_id, name=DEFAULT_AI_NAME, desc=DEFAULT_AI_DESC,
                                         model=DEFAULT_MODEL, exists_ok=True, tools=[CurrentContextToolDesc])
        cls.user = create_user(id=cls.user_id, name=DEFAULT_USER_NAME, desc=DEFAULT_USER_DESC, exists_ok=True)

    def test_assistant_chat(self):
        session_id = "ut_session"

        # 测试问答
        message: UserMessage = UserMessage(content="你好，我最近心情不好，不知道该怎么办", user_id=self.user_id, session_id=session_id)
        resp = self.assistant.chat(self.user_id, message, max_tokens=128)
        show_message(resp)

        # 测试流式
        message: UserMessage = UserMessage(content="能给我推荐三首歌么", user_id=self.user_id, session_id=session_id)
        resp = self.assistant.chat(self.user_id, message, max_tokens=128, stream=True, do_remember=True)
        show_message(resp)

        # 测试多轮
        message: UserMessage = UserMessage(content="详细介绍一下第三首", user_id=self.user_id, session_id=session_id)
        resp = self.assistant.chat(self.user_id, message, max_tokens=64)
        show_message(resp)

        # 测试工具调用
        message: UserMessage = UserMessage(content="今天是几号呀，星期几？", user_id=self.user_id, session_id=session_id)
        resp = self.assistant.chat(self.user_id, message, max_tokens=64)
        show_message(resp)

        # 清空session
        clear_session(session_id)

    def test_speak(self):
        assistant = get_assistant(self.assistant_id)
        message = AssistantMessage(content="你好，我叫Aifori, 有什么能帮到你的么？", user_id=self.assistant_id)
        voice: Voice = assistant.speak(message)
        play_voice(voice=voice)

    def test_recommend_music(self):
        tools = [MusicToolDesc]
        assistant = create_assistant(id="ut_tool_assistant", name="ut_tool_assistant",
                                     desc="ut_tool_assistant", model=DEFAULT_MODEL, exists_ok=True, tools=tools)

        resp = assistant.chat(self.user_id, UserMessage(content="推荐一首欢快的歌曲", user_id=self.user_id))
        logger.info(resp)
        show_message(resp)
